@@ -2,12 +2,16 @@
 from inspect import getfile
 from tinyscript.helpers import flatten_dict, BorderlessTable, Path, PathBasedDict
 
+from .components.logger import get_logger
 from .entity import Entity, MetaEntity
 
 
 __all__ = ["Module"]
 
 
+logger = get_logger("core.module")
+
+
 class MetaModule(MetaEntity):
     """ Metaclass of a Module. """
     _has_config       = True
